﻿//Copyright (C) 2007-2024 Mariano Ortu <https://www.speedcrypt.info/>

using System;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

using AES_StringCrypto.Blake;
using AES_StringCrypto.Random;
using AES_StringCrypto.Crypto;



namespace AES_StringCrypto
{
    public partial class FrmMain : Form
    {
        #region Constructors
        public FrmMain()
        {
            InitializeComponent();
            loadall();
        }

        #endregion Constructors

        #region Form Routines
        void loadall()
        {
            //Form Settings
            Text = "AES String Crypto...";
            StartPosition = FormStartPosition.CenterScreen;
            MaximizeBox = false;
            MinimizeBox = false;
            FormBorderStyle = FormBorderStyle.FixedDialog;
            labOrtu.Text = "Copyright (C) 2007 - " + DateTime.Now.ToString("yyyy") + " Mariano Ortu";

            //Menu
            infoMenuItem.Click += InfoMenuItem_Click;
            closeMenuItem.Click += CloseMenuItem_Click;

            //TextBox
            txtStpw.ReadOnly = true;
            txtBlk.ReadOnly = txtStpw.ReadOnly;
            txtNbr.ReadOnly = txtStpw.ReadOnly;
            txtSalt.ReadOnly = txtStpw.ReadOnly;
            txtRes.ReadOnly = txtStpw.ReadOnly;
            txtPsw.TextChanged += TxtPsw_TextChanged;

            //Buttons
            btnEnc.Enabled = false;
            btnDec.Enabled = btnEnc.Enabled;
            btnEnc.Cursor = Cursors.Hand;
            btnDec.Cursor = btnEnc.Cursor;
            btnEnc.Click += BtnEnc_Click;
            btnEnc.MouseMove += BtnEnc_MouseMove;
            btnEnc.MouseLeave += BtnEnc_MouseLeave;
            btnDec.Click += BtnDec_Click;
        }

        #endregion Form Routines

        #region Menu
        private void CloseMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void InfoMenuItem_Click(object sender, EventArgs e)
        {
            FrmInfo frm = new FrmInfo(this)
            {
                StartPosition = FormStartPosition.CenterScreen
            };
            frm.ShowDialog();
        }

        #endregion Menu

        #region TextBox

        private void TxtPsw_TextChanged(object sender, EventArgs e)
        {
            btnEnc.Enabled = false;
            btnEnc.Enabled = txtPsw.Text.Length > 5;
        }

        #endregion TextBox

        #region Buttons
        private void BtnEnc_Click(object sender, EventArgs e)
        {
            try
            {
                Rndm rndmz = new Rndm();
                //Blake 256
                Blake256 blake = new Blake256();
                txtNbr.Text = Convert.ToString(rndmz.Next(1, 20000000));
                txtSalt.Text = Convert.ToBase64String(Encoding.Default.GetBytes(Convert.ToString(txtNbr.Text)));
                txtStpw.Text = Convert.ToBase64String(Encoding.Default.GetBytes(txtPsw.Text));
                txtBlk.Text = BitConverter.ToString(blake.ComputeHash(Encoding.Default.GetBytes(txtPsw.Text + txtPsw.Text))).ToLower().Replace("-", "");
                txtRes.Text = Aes256CBCEncrypter.Encrypt(txtSalt.Text, BitConverter.ToString(blake.ComputeHash(Encoding.Default.GetBytes(txtPsw.Text))).ToLower().Replace("-", "").Substring(0, 32));
            }
            catch
            {
                //Silent Exception
                //At your discretion
            }
            finally
            {
                btnDec.Enabled = true;
                btnDec.Focus();
            }
        }
        private void BtnDec_Click(object sender, EventArgs e)
        {
            try
            {
                Blake256 blake = new Blake256();
                txtRes.Text = Aes256CBCEncrypter.Decrypt(txtRes.Text, BitConverter.ToString(blake.ComputeHash(Encoding.Default.GetBytes(txtPsw.Text))).ToLower().Replace("-", "").Substring(0, 32));
            }
            catch
            {
                //Silent Exception
                //At your discretion
            }
            finally
            {
                btnDec.Enabled = false;
                txtSalt.BackColor = Color.Yellow;
                txtRes.BackColor = txtSalt.BackColor;
                btnEnc.Focus();
            }
        }
        private void BtnEnc_MouseLeave(object sender, EventArgs e)
        {
            txtStpw.BackColor = Color.White;
            txtBlk.BackColor = txtStpw.BackColor;
            txtNbr.BackColor = txtStpw.BackColor;
            txtSalt.BackColor = txtStpw.BackColor;
            txtRes.BackColor = txtStpw.BackColor;
        }
        private void BtnEnc_MouseMove(object sender, MouseEventArgs e)
        {
            txtStpw.BackColor = Color.Yellow;
            txtBlk.BackColor = txtStpw.BackColor;
            txtNbr.BackColor = txtStpw.BackColor;
            txtSalt.BackColor = txtStpw.BackColor;
            txtRes.BackColor = txtStpw.BackColor;
        }

        #endregion Buttons
    }
}
