﻿//Copyright (C) 2007-2024 Mariano Ortu <https://www.speedcrypt.info/>

using System;

namespace AES_StringCrypto.Random
{
      public class Rndm
    {
        private int init;
        private int init_;
        private int[] SeedAr = new int[56];
        public Rndm()
            : this(Environment.TickCount)
        {
        }
        public Rndm(int Seed)
        {
            int num = ((Seed == int.MinValue) ? int.MaxValue : Math.Abs(Seed));
            int num_1 = 161803398 - num;
            SeedAr[55] = num_1;
            int num_2 = 1;
            for (int i = 1; i < 55; i++)
            {
                int num_3 = 21 * i % 55;
                SeedAr[num_3] = num_2;
                num_2 = num_1 - num_2;
                if (num_2 < 0)
                {
                    num_2 += int.MaxValue;
                }

                num_1 = SeedAr[num_3];
            }

            for (int j = 1; j < 5; j++)
            {
                for (int k = 1; k < 56; k++)
                {
                    SeedAr[k] -= SeedAr[1 + (k + 30) % 55];
                    if (SeedAr[k] < 0)
                    {
                        SeedAr[k] += int.MaxValue;
                    }
                }
            }

            init = 0;
            init_ = 21;
            Seed = 1;
        }
        protected virtual double Sample()
        {
            return (double)InternalSample() * 4.6566128752457969E-10;
        }
        private int InternalSample()
        {
            int num = init;
            int num_ = init_;
            if (++num >= 56)
            {
                num = 1;
            }

            if (++num_ >= 56)
            {
                num_ = 1;
            }

            int num_0 = SeedAr[num] - SeedAr[num_];
            if (num_0 == int.MaxValue)
            {
                num_0--;
            }

            if (num_0 < 0)
            {
                num_0 += int.MaxValue;
            }

            SeedAr[num] = num_0;
            init = num;
            init_ = num_;
            return num_0;
        }
        public virtual int Next()
        {
            return InternalSample();
        }
        private double GetSampleForLargeRange()
        {
            int num = InternalSample();
            if ((InternalSample() % 2 == 0) ? true : false)
            {
                num = -num;
            }

            double num_4 = num;
            num_4 += 2147483646.0;
            return num_4 / 4294967293.0;
        }
        public virtual int Next(int minValue, int maxValue)
        {
            if (minValue > maxValue)
            {
                                
                throw new ArgumentOutOfRangeException("ArgumentOutOfRange", "minValue", "maxValue");
            }

            long num = (long)maxValue - (long)minValue;
            if (num <= int.MaxValue)
            {
                return (int)(Sample() * (double)num) + minValue;
            }

            return (int)((long)(GetSampleForLargeRange() * (double)num) + minValue);
        }
        public virtual int Next(int maxValue)
        {
            if (maxValue < 0)
            {
                throw new ArgumentOutOfRangeException("ArgumentOutOfRange", "MustBePositive", "maxValue");
            }

            return (int)(Sample() * (double)maxValue);
        }
        public virtual double NextDouble()
        {
            return Sample();
        }

        public virtual void NextBytes(byte[] buffer)
        {
            if (buffer == null)
            {
                throw new ArgumentNullException("buffer");
            }

            for (int i = 0; i < buffer.Length; i++)
            {
                buffer[i] = (byte)(InternalSample() % 256);
            }
        }
    }
}
